function [SepWavFrame, SepHalfSpec] = SeperateAFrame_Simple(spec, SourceF0, TrackNum, para)
% Seperate a frame of the audio mixture into several source frames
%
% Input
%   - spec          : the half complex spectrum of the mixture frame
%   - SourceF0      : F0s of the mixture frame (MIDI numbers, the i-th one is for Track i, 0 if no F0 for that track)
%   - TrackNum      : the number of tracks
%   - para          
%       - fftLen    : fft length  
%       - fs        : sampling frequency
%       - harmOvlpTh: frequency deviation threshold of two harmonics to be
%       judged being non-overlapped. Possible values: 40Hz (since 20Hz is
%       -6dB decrease for hamming window of length 46ms); 60Hz (since 30Hz
%       is -6dB decrease for hamming window of length 32ms);
% Output
%   - SepWavFrame   : separated source frames (each column is a source)
%   - SepHalfSpec   : half complex spectrum of separated source (each
%                       column is a source
%
% Author: Zhiyao Duan
% Created: 9/17/2010
% Last modified: 10/29/2013

fftLen = para.fftLen;
fs = para.fs;
harmOvlpTh = para.harmOvlpTh;                                 
    
% allocate memory
SepWavFrame = zeros(fftLen, TrackNum);
SepHalfSpec = zeros(fftLen/2+1, TrackNum);
harmMask = zeros(fftLen/2, TrackNum);

% construct a mask for each track (F0)
for tnum = 1:TrackNum
    if SourceF0(tnum) == 0
        continue;
    end
    % find out the harmonic positions
    harmFreq = midi2hz(SourceF0(tnum)) : midi2hz(SourceF0(tnum)) : 16000;        % harmonic frequencies in Hz
    harmFreqL = harmFreq - harmOvlpTh/2;                    % harmonic left and right boundaries in Hz
    harmFreqR = harmFreq + harmOvlpTh/2;
    harmPos = round(harmFreq*fftLen/fs);                    % harmonic frequencies in points
    harmPosL = round(harmFreqL*fftLen/fs);                  % harmonic left and right boundaries in points
    harmPosR = round(harmFreqR*fftLen/fs);

    % make a triagular bump around the harmonic frequency between the left
    % and right boundaries as the masking value. This bump should cover the
    % range of -6dB drop.
    for hnum = 1:length(harmFreq)
        leftInterval = harmPos(hnum)-harmPosL(hnum);
        harmMask(harmPosL(hnum) : harmPos(hnum), tnum) = (1/2 : 1/2/leftInterval : 1) * 1/hnum^2;
        rightInterval = harmPosR(hnum)-harmPos(hnum);
        harmMask(harmPos(hnum) : harmPosR(hnum), tnum) = (1 : -1/2/rightInterval : 1/2) * 1/hnum^2;
    end
end

% normalize harmonic masks so that for each spectral bin the sum of masks is equal to 1
harmTotalMask = sum(harmMask, 2);
harmMask(harmTotalMask==0, :) = 1/TrackNum;
idx = harmTotalMask~=0;
harmMask(idx,:) = harmMask(idx,:) ./ repmat(harmTotalMask(idx,:), 1, TrackNum);
% reconstruct source wave
ObSpecMag = abs(spec);                                      % magnitude spectrum of the mixture
ObSpecPhase = angle(spec);                                  % phase spectrum of the mixture
for tnum = 1:TrackNum
    SrcSpec = zeros(fftLen, 1);
    SrcSpec(1) = 1/TrackNum * spec(1);
    SrcSpec(2:fftLen/2+1) = harmMask(:, tnum) .* ObSpecMag(2:fftLen/2+1) .* exp(1i*ObSpecPhase(2:fftLen/2+1));
    SrcSpec(fftLen/2+2 : end) = conj(SrcSpec(fftLen/2 : -1 : 2));
    SepWavFrame(:, tnum) = real(ifft(SrcSpec));
    SepHalfSpec(:, tnum) = SrcSpec(1:fftLen/2+1);
end
