function [testLab, testFea, testPitch] = createTestData_RWC(feaType, numMix, poly, maxHarm)
% Create test data for SVM from randomly mixed frames of isolated IOWA instrument notes
%
% Input
%   - feaType   : OC, MFCC, DC, RDC, UDC, MUDC, HS
%   - numMix    : the number of test frames
%   - poly      : the number of notes in each mixture
%   - maxHarm   : maximum number of harmonics to use (only valid for
%                   'DC', 'RDC', 'UDC', 'MUDC', and 'HS'. It determines the
%                   maximum order of cepstrum.)
% Output
%   - testLab  : column vector of class labels of training data
%   - testFea  : each row is an instance; each column is a feature
%   - testPitch: column vector of the pitch of each feature
%
% Author: Zhiyao Duan
% Created: 10/19/2013
% Last modified: 10/19/2013

%% Control parameters
% where the RWC dataset is located
FileDir = 'W:\Work\My_data\RWC\RWC_Instruments_SegmentedNotes\';

InstName = {...
    '33';...    % flute
    '29';...    % oboe
    '31';...    % clarinet
    '30';...    % bassoon
    '26';...    % alto saxophone
    '21';...    % trumpet
    '24';...    % horn
    '22';...    % trombone
    '23';...    % tuba
    '15';...    % violin
    '16';...    % viola
    '17';...    % cello
    '18';...    % bass
    };
    
instNum = length(InstName);

%% Other parameters
frameLen = 2048;                            % 46ms for fs=44100
zpf = 4;                                    % zero padding factor
fftLen = 2^nextpow2(frameLen*zpf);          % fft length (full spectrum length)
win = hamming(frameLen, 'periodic');        % window function
hop = 441;                                  % 10ms for fs=44100

harmOvlpTh = 40;                            % threshold (Hz) to determine if two harmonics are overlapping
p = maxHarm;                                % cepstrum order
N = fftLen/2 + 1;                           % half spectrum length

% peak detection parameters
para.peakTh = 50;                           % global amplitude threshold (dB)
para.peakTh_rel = 4;                        % local amplitude threshold (dB)
para.peakTh_freq = 3715;                    % frequency threshold (ppints)
para.movL = 74;                             % moving average width (points)
para.localRange = 3;                        % the range for considering local maximum

% allocate memory
testLab = zeros(numMix*poly, 1);
testFea = zeros(numMix*poly, p);
testPitch = zeros(numMix*poly, 1);

%%
for i = 1:numMix
    fprintf('Processing file No. %d...\n', i);
    wavMix = zeros(frameLen, 1);
    instMix = randsample(instNum, poly);
    for j = 1:poly
        % randomly select an instrument
        inst = instMix(j);

        % randomly select a note from the instrument
        wavFiles = dir(strcat(FileDir, InstName{inst}, '*NOM*.wav'));   % only use the non vibrato notes
        wavFileNum = length(wavFiles);
        wavIdx = randi(wavFileNum);

        % choose a frame
        fprintf('Taking %s...\n', fullfile(FileDir, wavFiles(wavIdx).name));
        [wavdata, fs] = audioread(fullfile(FileDir, wavFiles(wavIdx).name));
        % convert to mono
        wavdata = mean(wavdata, 2);
        
%         % randomly pick one frame for testing
%         nFrame = floor(length(wavdata)/frameLen);
%         wavRMS = zeros(1, nFrame);
%         for k = 1:nFrame
%             wavRMS(k) = rms(wavdata(frameLen*(k-1)+1 : frameLen*k));
%         end
%         idx = find(wavRMS > mean(wavRMS));
%         idxTrainFrame = randsample(idx, 1);
%         wavFrame = wavdata((idxTrainFrame-1)*frameLen+1 : idxTrainFrame*frameLen);

        % extract the wavform in the middle frame
        idx = round(length(wavdata)/2);
        wavFrame = wavdata(idx + 1: idx + frameLen);
        
        % normalize the frame
        wavFrame = wavFrame / rms(wavFrame);

        % call YIN algorithm to estimate the pitch
        P.sr = fs;
        P.wsize = length(wavFrame)/2;
        P.minf0 = 50;
        P.maxf0 = 2000;
        R = yin(wavFrame, P);
        tmpf0 = R.f0;
        [~, idx] = min(R.ap(1:round(end/2)) ./ R.pwr(1:round(end/2)));
        f0 = 2^ tmpf0(idx) * 440;
        
        % mix
        wavMix = wavMix + wavFrame;
        
        testLab((i-1)*poly+j) = inst;
        testPitch((i-1)*poly+j) = f0;
    end
    wavMix = wavMix/rms(wavMix);
    
    % calculate feature from the mixed frame
    % zero padding, windowing, fft, taking the first half of the spectrum
    spec_com = fft(wavMix.*win, fftLen);
    spec = 20*log10(abs(spec_com));
    spec = spec(1:N);
    
    % separate the mixture spectrum
    if strcmp(feaType, 'OC') || strcmp(feaType, 'MFCC')
        % Separate the mixture
        para= {};
        para.fftLen = fftLen;
        para.fs = fs;
        para.harmOvlpTh = 40;   % 40Hz
        F0s = hz2midi(testPitch((i-1)*poly+1 : (i-1)*poly+poly));
        [SepWavFrame, SepHalfSpec] = SeperateAFrame_Simple(spec_com(1:fftLen/2+1), F0s, poly, para);
        SepWavFrame = SepWavFrame(1:frameLen, :);
    end
    
    % find non-overlap harmonics
    NonOvlpHarm = FindNonOverlapHarmonics(testPitch((i-1)*poly+1:(i-1)*poly+poly), maxHarm, harmOvlpTh);
    
    % calculate feature for each pitch
    for j = 1:poly
        f0 = testPitch((i-1)*poly+j);
        % peak detection
        if strcmp(feaType, 'DC') || strcmp(feaType, 'RDC') || strcmp(feaType, 'UDC') || strcmp(feaType, 'MUDC') || strcmp(feaType, 'HS')
            [peak, peakNum, ~] = PeakExtract(spec, para, 0);        % spectral peaks
            [peak, peakAmp] = PeakInterpolate(spec, peak);          % refine the location and amplitudes of spectral peaks
               
            if strcmp(feaType, 'DC') || strcmp(feaType, 'RDC') || strcmp(feaType, 'UDC') || strcmp(feaType, 'MUDC')
                % only keep the peaks that correspond to non-overlapping
                % harmonics. This is performed only for DC, RDC, and UDC,
                % because in the HS calcutation fuction a similar step is
                % performed
                fpeak = (peak-1)*fs/fftLen;
                RR = round(fpeak/f0);
                idx = (abs(fpeak/f0 - RR) < 0.03) & (RR~=0) & (RR<=maxHarm);
                if ~isempty(peak(idx))                    
                    peak = peak(idx);
                    fpeak = fpeak(idx);
                    peakAmp = peakAmp(idx);
                    % do not include peaks that are very far from previous
                    % peaks
                    idx2 = [true; diff(RR(idx))<4];
                    peak = peak(idx2);
                    peakAmp = peakAmp(idx2);
                end
                % associate each harmonic only one peak, the highest peak
                [peakAmp, idx] = sort(peakAmp, 'descend');
                peak = peak(idx);
                fpeak = fpeak(idx);
                RR = round(fpeak/f0);
                [RR, idx] = unique(RR);
                peak = peak(idx);
                peakAmp = peakAmp(idx);
                
%                 % only use the non-overlapping harmonics
%                 idx = false(size(RR));
%                 for h = 1:length(RR)
%                     idx(h) = any(NonOvlpHarm(:,j)==RR(h));
%                 end
%                 % if there are enough non-overlapping harmonics
%                 if sum(idx) > 5
%                     peak = peak(idx);
%                     peakAmp = peakAmp(idx);                
%                 end
                
                % get normalized frequency
                f = (peak-1)/fftLen;                                        % normalized frequency
                a = peakAmp;
            end
        end
        
        switch feaType
            case 'OC'
                f = (0 : 1/(2*N) : 1/2-1/(2*N))';           % normalized frequency of frequency bins
                M = zeros(N, p);                            % DCT matrix
                M(:,1) = 1;
                M(:,2:end) = sqrt(2) * cos(2*pi * f * (1:p-1));                
                tempspec = fft(SepWavFrame(:,j), fftLen);
                temp = M' * 20*log10(abs(tempspec(1:fftLen/2+1)));
                testFea((i-1)*poly+j,:) = temp';
            case 'MFCC'
                temp = melfcc(SepWavFrame(:,j), fs, 'wintime', frameLen/fs, 'numcep', p);
                testFea((i-1)*poly+j,:) = temp';   
            case 'DC'
                temp = disceps(f, a, p, 0);
                testFea((i-1)*poly+j, :) = temp';
            case 'RDC'
                temp = disceps(f, a, p);                             % the regularization parameter is not set, hence the default 1e-4 will be used.
                testFea((i-1)*poly+j, :) = temp';
            case 'UDC'
                temp = unidisceps(f, a, p);
                testFea((i-1)*poly+j, :) = temp';
            case 'MUDC'
                temp = unidisceps(f, a, p, fs, 1);
                testFea((i-1)*poly+j, :) = temp';
            case 'HS'
                fpeak = (peak-1)*fs/fftLen;
                temp = harmstruct(f0, fpeak, peakAmp, maxHarm);
                testFea((i-1)*poly+j, :) = temp';   
        end
    end
end
