function [trainLab, trainFea] = createTrainData(feaType, maxHarm, nTrainFrame)
% Create training data for SVM from isolated IOWA instrument notes
%
% Input
%   - feaType   : OC, MFCC, DC, RDC, UDC, MUDC HS
%   - maxHarm   : maximum number of harmonics to use (only valid for
%                   'DC', 'RDC', 'UDC', 'MUDC', and 'HS'. It determines the
%                   maximum order of cepstrum.)
%   - nFrame    : number of frames chosed for training
% Output
%   - trainLab  : column vector of class labels of training data
%   - trainFea  : each row is an instance; each column is a feature
%
% Author: Zhiyao Duan
% Created: 8/19/2013
% Last modified: 10/30/2013

%% Control parameters
% where the IOWA dataset is located
FileDir = 'W:\Work\My_data\ChordData\SampleMaterial\SegmentedDataAll_mf\';

InstName = {...
    'Flute';...
    'Oboe';...
    'BbClar';...
    'Bassoon';...
    'AltoSax';...
    'Trumpet';...
    'Horn';...
    'TenorTrombone';...
    'Tuba';...
    'Violin_arco';...
    'Viola_arco';...
    'Cello_arco';...
    'Bass_arco';...
    };
    
InstNum = length(InstName);

%% Other parameters
frameLen = 2048;                            % 46ms for fs=44100
zpf = 4;                                    % zero padding factor
fftLen = 2^nextpow2(frameLen*zpf);          % fft length (full spectrum length)
win = hamming(frameLen, 'periodic');        % window function
hop = 441;                                  % 10ms for fs=44100

p = maxHarm;                                % the maximum cepstral order is the same as the maximum harmonic number
N = fftLen/2 + 1;                           % half spectrum length

% peak detection parameters
para.peakTh = 50;                           % global amplitude threshold (dB)
para.peakTh_rel = 4;                        % local amplitude threshold (dB)
para.peakTh_freq = 3715;                    % frequency threshold (ppints)
para.movL = 74;                             % moving average width (points)
para.localRange = 3;                        % the range for considering local maximum

% allocate memory
trainNum = 0;
for inst = 1:InstNum
    wavFiles = dir(fullfile(FileDir, InstName{inst}, '*.wav'));
    trainNum = trainNum + length(wavFiles);
end
trainLab = zeros(trainNum*nTrainFrame, 1);
trainFea = zeros(trainNum*nTrainFrame, p);

%% Process audio
currTrain = 0;
for inst = 1:InstNum
    wavFiles = dir(fullfile(FileDir, InstName{inst}, '*.wav'));
    wavFileNum = length(wavFiles);
        
    for i = 1:wavFileNum
        % wave file
        fprintf('Processing %s...\n', fullfile(FileDir, InstName{inst}, wavFiles(i).name));
        [wavdata, fs] = audioread(fullfile(FileDir, InstName{inst}, wavFiles(i).name));
        
        % convert to mono
        wavdata = mean(wavdata, 2);
        
        % randomly pick nFrame frames for training
        nFrame = floor(length(wavdata)/frameLen);
        wavRMS = zeros(1, nFrame);
        for k = 1:nFrame
            wavRMS(k) = rms(wavdata(frameLen*(k-1)+1 : frameLen*k));
        end
        idx = find(wavRMS > mean(wavRMS));
        idxTrainFrame = randsample(idx, nTrainFrame);
        
        for j = 1:nTrainFrame
            wavFrame = wavdata((idxTrainFrame(j)-1)*frameLen+1 : idxTrainFrame(j)*frameLen);
    
            wavFrame = wavFrame / rms(wavFrame);

            % zero padding, windowing, fft, taking the first half of the spectrum
            spec = 20*log10(abs(fft(wavFrame.*win, fftLen)));
            spec = spec(1:N);

            % peak detection
            if strcmp(feaType, 'DC') || strcmp(feaType, 'RDC') || strcmp(feaType, 'UDC') || strcmp(feaType, 'MUDC') || strcmp(feaType, 'HS')
                [peak, peakNum, ~] = PeakExtract(spec, para, 0);        % spectral peaks
                [peak, peakAmp] = PeakInterpolate(spec, peak);          % refine the location and amplitudes of spectral peaks

                % find pitch range
                if strcmp(InstName{inst}, 'Piano')
                    [f0min, f0max] = estf0range2(wavFiles(i).name);
                else
                    [f0min, f0max] = estf0range(wavFiles(i).name);
                end

                % call YIN algorithm to estimate the pitch
                P.sr = fs;
                P.wsize = length(wavFrame)/2;
                P.minf0 = f0min;
                P.maxf0 = f0max;
                R = yin(wavFrame, P);
                tmpf0 = R.f0;
                [~, idx] = min(R.ap(1:round(end/2)) ./ R.pwr(1:round(end/2)));
                f0 = 2^ tmpf0(idx) * 440;

                if strcmp(feaType, 'DC') || strcmp(feaType, 'RDC') || strcmp(feaType, 'UDC') || strcmp(feaType, 'MUDC')
                    % only keep the peaks that correspond to harmonics
                    fpeak = (peak-1)*fs/fftLen;
                    RR = round(fpeak/f0);
                    idx = (abs(fpeak/f0 - RR) < 0.03) & (RR~=0) & (RR<=maxHarm);
                    % only take the harmonics if harmonics are correctly found,
                    % otherwise use all peaks
                    if ~isempty(peak(idx))   
                        peak = peak(idx);
                        fpeak = fpeak(idx);
                        peakAmp = peakAmp(idx);
                    end
                    % associate each harmonic only one peak, the highest peak
                    [peakAmp, idx] = sort(peakAmp, 'descend');
                    peak = peak(idx);
                    fpeak = fpeak(idx);
                    RR = round(fpeak/f0);
                    [RR, idx] = unique(RR);
                    peak = peak(idx);
                    peakAmp = peakAmp(idx);
                    
                    % get normalized frequency
                    f = (peak-1)/fftLen;                                        % normalized frequency
                    a = peakAmp;
                end            
            end
        
            currTrain = currTrain + 1;
            trainLab(currTrain) = inst;                                 % class label
            switch feaType
                case 'OC'
                    f = (0 : 1/(2*N) : 1/2-1/(2*N))';           % normalized frequency of frequency bins
                    M = zeros(N, p);                            % DCT matrix
                    M(:,1) = 1;
                    M(:,2:end) = sqrt(2) * cos(2*pi * f * (1:p-1));

                    temp = M' * spec;
                    trainFea(currTrain,:) = temp';
                case 'MFCC'
                    temp = melfcc(wavFrame, fs, 'wintime', frameLen/fs, 'numcep', p);
                    trainFea(currTrain,:) = temp';   
                case 'DC'
                    temp = disceps(f, a, p, 0);
                    trainFea(currTrain, :) = temp';
                case 'RDC'
                    temp = disceps(f, a, p);                             % the regularization parameter is not set, hence the default 1e-4 will be used.
                    trainFea(currTrain, :) = temp';
                case 'UDC'
                    temp = unidisceps(f, a, p);
                    trainFea(currTrain, :) = temp';
                case 'MUDC'
                    temp = unidisceps(f, a, p, fs, 1);
                    trainFea(currTrain, :) = temp';
                case 'HS'
                    fpeak = (peak-1)*fs/fftLen;
                    temp = harmstruct(f0, fpeak, peakAmp, maxHarm);
                    trainFea(currTrain,:) = temp';   
            end
        end
    end
end
    

